//*****************************************************************************
//
//	I2C - Bus Routinen Grundmodul
//
//*****************************************************************************

#ifndef __I2C__
#define __I2C__

#define I2C_TWIE               0x01
#define I2C_TWEN               0x04
#define I2C_TWWC               0x08
#define I2C_TWSTOP             0x10
#define I2C_TWSTART            0x20
#define I2C_TWACK              0x40
#define I2C_TWINT              0x80

//Master
#define MSTAT_START            0x08
#define MSTAT_REPEAT_START     0x10
#define MSTAT_WSLA_ACK         0x18
#define MSTAT_WSLA_NACK        0x20
#define MSTAT_WDB_ACK          0x28
#define MSTAT_WDB_NACK         0x30
#define MSTAT_LOST             0x38
#define MSTAT_RSLA_ACK         0x40
#define MSTAT_RSLA_NACK        0x48
#define MSTAT_RDB_ACK          0x50
#define MSTAT_RDB_NACK         0x58

//Slave
#define SSTAT_WSLA_ACK         0x60
#define SSTAT_WSLA_NACK        0x68
#define SSTAT_GCALL            0x70
#define SSTAT_LOST             0x78 
#define SSTAT_PWSLA_ACK        0x80
#define SSTAT_PWSLA_NACK       0x88
#define SSTAT_RDBGCALL_ACK     0x90
#define SSTAT_RDBGCALL_NACK    0x98
#define SSTAT_REPEAT_START     0xA0
#define SSTAT_RSLA_ACK         0xA8
#define SSTAT_RSLA_NACK        0xB0
#define SSTAT_RDB_ACK          0xB8
#define SSTAT_RDB_NACK         0xC0
#define SSTAT_RLASTDB_ACK      0xC8

#define I2C_STATE_CLEAR        0x00
#define I2C_STATE_CONSTART     0x01
#define I2C_STATE_TBADR        0x02
#define I2C_STATE_TBDB         0x04
#define I2C_STATE_RB           0x08
#define I2C_STATE_RBEND        0x10
#define I2C_STATE_CONSTOP      0x20


//Fkt. fuer Ansteuerung der I2C-Bausteine
void I2C_init(void);
void I2C_constart(void);
void I2C_constop(void);
void I2C_transbyteADR(unsigned char uc_byte);
void I2C_transbyte(unsigned char uc_byte);
unsigned char I2C_recbyte(void);
unsigned char I2C_recbyteend(void);
void I2C_BUSSTOP(void);

//interne Fkt.
unsigned char I2C_GetByte(unsigned char uc_lastbyte);
void I2C_wait(void);
unsigned char I2C_ERROR(void);

#endif