/*****************************************************************************/
//
//  Routinen fuer das Lesen und Schreiben ueber I2C-Bus
//
//  Takt = 400kHz, ohne Bitstretching
/*****************************************************************************/

#include <iom128v.h>
#include "I2CH.h"
#include "i2ch_eep.h"

unsigned char uc_ADR_BYTE = 2;


/*****************************************************************************/
//  Setze 1 oder 2 ADR-Byte
//  Input:  Anzahl
//  Output: -
/*****************************************************************************/

void eep_adrbyte(unsigned char uc_anzahl)
{
   if ((uc_anzahl == 1) || (uc_anzahl == 2))  uc_ADR_BYTE = uc_anzahl;  
}


/*****************************************************************************/
//  Byte in EEPROM schreiben
//  Input:  I2C-Adr,Speicherzelle, DB
//  Output: -
/*****************************************************************************/

void write_promb(unsigned char uc_adr,unsigned int ui_zelle,unsigned char uc_daten)
{
   I2C_constart();
   if (I2C_ERROR()) return; 
   I2C_transbyteADR(uc_adr & 0xFE);
   if (I2C_ERROR()) return; 
   if (uc_ADR_BYTE == 2)
    {
       I2C_transbyte((unsigned int) ui_zelle >> 8);  //HWT ADDRESS  
    }
   if (I2C_ERROR()) return;
   I2C_transbyte(ui_zelle);                          //NWT ADDRESS
   if (I2C_ERROR()) return;
   I2C_transbyte(uc_daten);                          //    DATEN
   if (I2C_ERROR()) return;
   I2C_constop();
   if (I2C_ERROR()) return;   
}


/*****************************************************************************/
//  Byte aus EEPROM lesen
//  Input:  I2C-Adr,Speicherzelle
//  Output: DB
/*****************************************************************************/

unsigned char read_promb(unsigned char uc_adr,unsigned int ui_zelle)
{
unsigned char uc_x = 0;

   I2C_constart();
   if (I2C_ERROR()) return(0);
   I2C_transbyteADR(uc_adr & 0xfe);
   if (I2C_ERROR()) return(0);
   if (uc_ADR_BYTE == 2)
    {
       I2C_transbyte((unsigned int) ui_zelle >> 8);  //HWT ADDRESS
    }
   if (I2C_ERROR()) return(0);
   I2C_transbyte(ui_zelle);                          //NWT ADDRESS
   if (I2C_ERROR()) return(0);
   I2C_constart();
   if (I2C_ERROR()) return(0);
   I2C_transbyteADR(uc_adr | 1);
   if (I2C_ERROR()) return(0);
   uc_x = I2C_recbyteend();
   if (I2C_ERROR()) return(0);
   I2C_constop();
   if (I2C_ERROR()) return(0);
   return(uc_x);
}


/*****************************************************************************/
//  Word in EEPROM schreiben
//  Input:  I2C-Adr,Speicherzelle, DW
//  Output: -
/*****************************************************************************/

void write_promw(unsigned char uc_adr,unsigned int ui_zelle,unsigned int ui_daten)
{
   I2C_constart();
   if (I2C_ERROR()) return; 
   I2C_transbyteADR(uc_adr & 0xFE);
   if (I2C_ERROR()) return; 
   if (uc_ADR_BYTE == 2)
    {
       I2C_transbyte((unsigned int) ui_zelle >> 8);  //HWT ADDRESS
    }
   if (I2C_ERROR()) return; 
   I2C_transbyte(ui_zelle);                          //NWT ADDRESS
   if (I2C_ERROR()) return; 
   I2C_transbyte((unsigned int) ui_daten >> 8);      //HWT DATEN
   if (I2C_ERROR()) return; 
   I2C_transbyte(ui_daten);                          //NWT DATEN
   if (I2C_ERROR()) return; 
   I2C_constop();
   if (I2C_ERROR()) return; 
}


/*****************************************************************************/
//  Word aus EEPROM lesen
//  Input:  I2C-Adr,Speicherzelle
//  Output: DW
/*****************************************************************************/

unsigned int read_promw(unsigned char uc_adr,unsigned int ui_zelle)
{
   unsigned int ui_x = 0;

   I2C_constart();
   if (I2C_ERROR()) return(0); 
   I2C_transbyteADR(uc_adr & 0xfe);
   if (I2C_ERROR()) return(0); 
   if (uc_ADR_BYTE == 2)
    {
       I2C_transbyte((unsigned int) ui_zelle >> 8);  //HWT ADDRESS
    }
   if (I2C_ERROR()) return(0); 
   I2C_transbyte(ui_zelle);                          //NWT ADDRESS
   if (I2C_ERROR()) return(0); 
   I2C_constart();
   if (I2C_ERROR()) return(0); 
   I2C_transbyteADR(uc_adr | 1);
   if (I2C_ERROR()) return(0); 
   ui_x = I2C_recbyte();                             //HWT DATEN
   if (I2C_ERROR()) return(0); 
   ui_x = ui_x << 8;
   ui_x |= I2C_recbyteend();                         //NWT DATEN
   if (I2C_ERROR()) return(0); 
   I2C_constop();
   if (I2C_ERROR()) return(0); 
   return(ui_x);
}


/*****************************************************************************/
//  DWord in EEPROM schreiben
//  Input:  I2C-Adr,Speicherzelle, DW
//  Output: -
/*****************************************************************************/

void write_proml(unsigned char uc_adr,unsigned int ui_zelle,unsigned long ul_daten)
{
unsigned int ui_data;

   I2C_constart();
   if (I2C_ERROR()) return; 
   I2C_transbyteADR(uc_adr & 0xFE);
   if (I2C_ERROR()) return; 
   if (uc_ADR_BYTE == 2)
    {
       I2C_transbyte((unsigned int) ui_zelle >> 8);  //HWT ADDRESS
    }
   if (I2C_ERROR()) return; 
   I2C_transbyte(ui_zelle);                          //NWT ADDRESS
   if (I2C_ERROR()) return;   
   ui_data = ul_daten >> 16;
   I2C_transbyte((unsigned int) ui_data >> 8);       //HWT DATEN
   if (I2C_ERROR()) return; 
   I2C_transbyte(ui_data);                           //NWT DATEN
   if (I2C_ERROR()) return; 
   ui_data = (unsigned int) ul_daten;
   I2C_transbyte((unsigned int) ui_data >> 8);       //HWT DATEN
   if (I2C_ERROR()) return; 
   I2C_transbyte(ui_data);                           //NWT DATEN
   if (I2C_ERROR()) return; 
   I2C_constop();
   if (I2C_ERROR()) return; 
}


/*****************************************************************************/
//  Word aus EEPROM lesen
//  Input:  I2C-Adr,Speicherzelle
//  Output: longwert
//zu grosse uL-Werte werden mit return(..) Rueckgabe verfaelscht
/*****************************************************************************/
unsigned long read_proml(unsigned char uc_adr,unsigned int ui_zelle,unsigned long * pul_wert)
{
unsigned long ul_wert;

  ul_wert = (unsigned long) read_promw(uc_adr,ui_zelle);
  ul_wert = ul_wert << 8; 
  ul_wert = ul_wert << 8; 
  ul_wert |= read_promw(uc_adr,ui_zelle+2);
 
//ul_wert = 180000000; //TEST dieser Wert laesst sich nicht mit return uebertragen

  if (pul_wert != 0x00) //Pointer gueltig
     *pul_wert = ul_wert;
  return(ul_wert);  
}


