//*****************************************************************************
//
//	Kommandointerpreter fuer UART0
//
//*****************************************************************************
#include <iom128v.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "kommand0.h"
#include "watchdog.h"
#include "proto0.h"
#include "spibus.h"
#include "thermo.h"
#include "ports.h"
#include "motor.h"

extern unsigned char uc_referenzfahrt,uc_referenzrichtung,uc_richtung;


char c_txstr[128];


//*****************************************************************************
//	Kommand - Kommandointerpreter
//  Input:	Befehlsstring
//	Output:	-
//*****************************************************************************
void Kommandointerpreter(char * pc_str)
{
unsigned char uc_ok = 0;

   watchdog_reset();
   if (strncmp(pc_str,"START",5) == 0)    uc_ok = BEFEHL_START(pc_str+5); 
   if (strncmp(pc_str,"STOP",4) == 0)     uc_ok = BEFEHL_STOP(pc_str+4); 
   if (strncmp(pc_str,"Z",1) == 0)        uc_ok = BEFEHL_ZUSTAND(pc_str+1); 
   if (strncmp(pc_str,"POS",3) == 0)      uc_ok = BEFEHL_POS(pc_str+3); 
   watchdog_reset();
   if (strncmp(pc_str,"VMIN",4) == 0)     uc_ok = BEFEHL_VMIN(pc_str+4); 
   if (strncmp(pc_str,"VMAX",4) == 0)     uc_ok = BEFEHL_VMAX(pc_str+4); 
   if (strncmp(pc_str,"V",1) == 0)        uc_ok = BEFEHL_GESCHWINDIGKEIT(pc_str+1); 
   if (strncmp(pc_str,"SV",2) == 0)       uc_ok = BEFEHL_SETGESCHWINDIGKEIT(pc_str+2); 
   if (strncmp(pc_str,"KP",2) == 0)       uc_ok = BEFEHL_KP(pc_str+2); 
   if (strncmp(pc_str,"MODE",4) == 0)     uc_ok = BEFEHL_STEPS(pc_str+4); 
   watchdog_reset();   
   if (strncmp(pc_str,"REF",3) == 0)      uc_ok = BEFEHL_REFERENZ(pc_str+3); 
   if (strncmp(pc_str,"RICHTUNG",8) == 0) uc_ok = BEFEHL_REFERENZRICHTUNG(pc_str+8); 

   if (strncmp(pc_str,"T",1) == 0)      uc_ok = BEFEHL_TEMP(pc_str+1);    
   if (strncmp(pc_str,"X",1) == 0)      uc_ok = BEFEHL_S1(pc_str+1);    
   if (strncmp(pc_str,"Y",1) == 0)      uc_ok = BEFEHL_S2(pc_str+1);    
      
   if (!uc_ok) SENDE_ERROR();
}


 
//*****************************************************************************
//	Kommand - Kommando Motor Start
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_START(char * pc_str)
{
unsigned long ul_pos;

  ul_pos = atol(pc_str);  
  if (MOTOR_DRIVE(ul_pos)) strcpy(c_txstr,"OK"); 
   else strcpy(c_txstr,"DRIVE ERROR");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Referenzfahrt
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_REFERENZ(char * pc_str)
{
  MOTOR_REFERENZFAHRT();
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Referenzpunkt links oder rechts
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_REFERENZRICHTUNG(char * pc_str)
{
  if (pc_str[0] == 0x30) MOTOR_REFERENZRICHTUNG(0); 
                    else MOTOR_REFERENZRICHTUNG(1); 
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Motor Stop
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_STOP(char * pc_str)
{
  MOTOR_OFF();
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Schrittweite Voll,halb,viertel,achtel
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_STEPS(char * pc_str)
{
unsigned int ui_mode;

  ui_mode = atoi(pc_str);
  MOTOR_MODE(ui_mode);
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Betriebszustand
//  Input:	-
//	Output:	-  0 steht  !=0 Motor in Fahrt
//*****************************************************************************
unsigned char BEFEHL_ZUSTAND(char * pc_str)
{
  sende_WertI(MOTOR_ZUSTAND());
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Get Position
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_POS(char * pc_str)
{
  sende_WertL(MOTOR_POSITION());
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Istgeschwindigkeit
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_GESCHWINDIGKEIT(char * pc_str)
{
  sende_WertI(MOTOR_GETDREHZAHL());
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Solldrehzahl
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_SETGESCHWINDIGKEIT(char * pc_str)
{
unsigned int ui_v;

  ui_v = atoi(pc_str);
  MOTOR_DREHZAHL(ui_v);
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Rampensteilheit 1... 25000
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_KP(char * pc_str)
{
unsigned int ui_param;

  ui_param = atoi(pc_str);
  MOTOR_PARAMETER_KP(ui_param);
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Minimalgeschwindigkeit
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_VMIN(char * pc_str)
{
unsigned int ui_param;

  ui_param = atoi(pc_str);
  MOTOR_PARAMETER_VMIN(ui_param);
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Maximalgeschwindigkeit
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_VMAX(char * pc_str)
{
unsigned int ui_param;

  ui_param = atoi(pc_str);
  MOTOR_PARAMETER_VMAX(ui_param);
  strcpy(c_txstr,"OK");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando TEMPERATUR
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_TEMP(char * pc_str)
{
  sende_WertI(TEMP_temperatur(0));
  return(1);
}

 
 
//*****************************************************************************
//	Kommand - Kommando Sollposition 0 Testfkt.
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_S1(char * pc_str)
{
  if (MOTOR_DRIVE(0)) strcpy(c_txstr,"OK"); 
   else strcpy(c_txstr,"DRIVE ERROR");
  sende_UART0(c_txstr);  
  return(1);
}


//*****************************************************************************
//	Kommand - Kommando Sollposition 5000 Testfkt.
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_S2(char * pc_str)
{
  if (MOTOR_DRIVE(5000)) strcpy(c_txstr,"OK"); 
   else strcpy(c_txstr,"DRIVE ERROR");
  sende_UART0(c_txstr);  
  return(1);
}


//*****************************************************************************
//	Kommand - Sende String an PC
//  Input:	-
//	Output:	-
//*****************************************************************************
void SENDE_ERROR(void)
{
  watchdog_reset();
  strcpy(c_txstr,"ERROR");
  sende_UART0(c_txstr);
}


