//*****************************************************************************
//
//	Steppermotor
//
//*****************************************************************************

#include <iom128v.h>
#include "m128int.h"
#include "motor.h"
#include "ports.h"
#include "timer.h"
#include "watchdog.h"

//#include "proto0.h"
//#include "kommand0.h"

#pragma interrupt_handler MOTOR_int:TIM2_OVF

#define PROZESSOR_CLK  16000000 //MHz
#define STEPS_U        200      //Schritte pro Umdrehung
#define VMIN           300      //Startgeschwindigkeit > 150min-1
#define VMAX           1000     //Maximalgeschwindigkeit
#define KP             24000    //Rampensteilheit 1...25000
#define VREF           150      //Referenzgeschwindigkeit 150min-1   

long sl_bremsrampe,sl_stepcount,sl_sollpos,sl_delta;
unsigned char uc_referenzfahrt,uc_referenzrichtung,uc_richtung,uc_notaus;
unsigned char uc_motorstate,uc_step,uc_vorteiler,uc_togglebit;
unsigned int ui_kp,ui_kp1,ui_rampe,ui_vmax,ui_vmin,ui_vsoll,ui_precount,ui_vist;
unsigned int ui_stepsproumdrehung,ui_vsollwert;
unsigned char TCLK = 0x00;
unsigned char TPRE = 0x03;
unsigned char uc_refstatus;
unsigned char uc_endlage = 0;
unsigned int ui_drive_cnt;


//*****************************************************************************
//	MOTOR - Timer2 Int  0,25ms (4kHz)
//  Input:	-
//	Output:	-
//*****************************************************************************
void MOTOR_int(void)
{
   TCCR2 = TPRE;
   TCNT2 = TCLK;
   TIFR |= 0x40;	//Overflow Flag reset
   
   //VORTEILER
   if (uc_vorteiler) uc_vorteiler --;
   if (uc_vorteiler == 0)
    {
	   switch(uc_step)
	   {
	   case 1:  uc_vorteiler = 8;break; //Vollschritt
	   case 2:  uc_vorteiler = 4;break; //Halbschritt
	   case 4:  uc_vorteiler = 2;break; //Viertelschritt
	   case 8:  uc_vorteiler = 1;break; //Achtelschritt
	   default: uc_vorteiler = 8;break; //Vollschritt	  
	   }

      //ab hier 1kHz Takt fuer alle Schrittweiten
	  if(!uc_referenzfahrt)
	   {
          if (uc_motorstate == 1) //Anfahren
    	   {		
	          if (sl_bremsrampe == sl_stepcount) uc_motorstate = 2;	
    	   }
	 		
          if (sl_stepcount == sl_sollpos)
           {
              //Stillstand nach bremsen
              MOTOR_BRAKE(1);
              PORTS_POWER(0);
	          uc_motorstate = 0;
              TCLK = 0x00;
              TPRE = 0x03;  
           }			
       }
	   else {  //Referenzfahrt - Aktivitaet
               TCCR2 = TPRE;
               TCNT2 = TCLK;

               switch(uc_refstatus)
			   {
			   case 0: break; //Nichts
			   case 1: //Aus der Endlagenposition herausfahren
                       if ((PINE & 0x60) != 0) 
 	                    {  //Einer der Endlagenschalter ist gesetzt
						
						   //ist der linke Endlagenschalter gesetzt?
                           if ((PINE & 0x20) == 0x20) uc_endlage = 1; 
						    else uc_endlage = 0;
                           
						   //ist der Referenzpunkt rechts, dann die Richtung 
						   //entgegengesetzt setzen        
                    	   if (uc_referenzrichtung)
                    		{
                               if (uc_endlage) MOTOR_RICHTUNG(0); 
							    else MOTOR_RICHTUNG(1);
                    		}
                     		else {
                    		        if (uc_endlage) MOTOR_RICHTUNG(1); 
									 else MOTOR_RICHTUNG(0);
                                 }
                           //Motor starten
                           ui_vsoll = MOTOR_Drehzahl2INC(VREF);
                           MOTOR_ON();
                           MOTOR_BRAKE(0);
                           uc_motorstate = 1;   					    	 
				           uc_refstatus = 2; //Warten bis herausgefahren
						}
						else uc_refstatus = 4; //steht nicht in der Endlage
					   break;  
               case 2: //Warten bis aus der Endlage herausgefahren wurde
			           if ((PINE & 0x60) != 0) uc_refstatus = 3;//noch weiterfahren
					   ui_drive_cnt = 0x200; //200 Schritte fahren
                       break;
			   case 3: //weiterfahren - Entprellen der Endlagenschalter		   
					   if (ui_drive_cnt > 0) ui_drive_cnt--;
					   if (ui_drive_cnt == 0) uc_refstatus = 4; //Start Referenzfahrt
                       break;
			   case 4: //Referenzfahrt starten	
 			           uc_motorstate = 0;		                       
                       uc_referenzfahrt = 1;	
                       MOTOR_RICHTUNG(0);
                       sl_stepcount = 0;	 	 
                       sl_sollpos = 0;

                       //Motor starten					   
                       ui_vsoll = MOTOR_Drehzahl2INC(VREF);
                       MOTOR_ON();
                       MOTOR_BRAKE(0);
                       uc_motorstate = 1;   					    	 
                       uc_refstatus = 5;  //suche REFERENZPUNKT
					   break;
			   case 5: //Referenzpunkt finden
			           if (uc_referenzfahrt == 2) uc_refstatus = 6; //Nullposition
					   break;
			   case 6: //Nullposition anfahren		    		   
                       //nach gefundenem Referenzpunkt um 200 Schritte entfernen 
					   //und dies nun als neuen Nullpunkt (Referenzpunkt fr 
					   //Antrieb) setzen   
                       uc_motorstate = 0;
                       uc_referenzfahrt = 1;
                       sl_stepcount = 0;	
                       sl_sollpos = 200;
             	       if (sl_sollpos < sl_stepcount) MOTOR_RICHTUNG(0); 
	                    else MOTOR_RICHTUNG(1);

                       //Motor starten													 
                       ui_vsoll = MOTOR_Drehzahl2INC(VREF);
                       MOTOR_ON();
                       MOTOR_BRAKE(0);
                       uc_motorstate = 1;   					    	 
                       uc_refstatus = 7; //Warten auf Erreichen der Zielposition 
					   break;
			   case 7: //Zielposition erreicht		   
					   if (sl_sollpos == sl_stepcount)
					    {
		                   //Ende der Referenzfahrt		      
						   uc_refstatus = 0;
                           uc_motorstate = 0;
                           uc_referenzfahrt = 0;
                           sl_stepcount = 0;
                           sl_sollpos = 0;
                           uc_notaus = 0;	
						}  
					   break;
			   default: break;		   
               }//case
	        }
	   	  
	  if (uc_motorstate) 
	   {
	      if (uc_togglebit)
		   {
  	         PORTS_CLK(1);
  	         MOTOR_STEPCOUNT();
		   }	 
   	       else PORTS_CLK(0);
		  uc_togglebit = ~uc_togglebit; 		  
	   	  
  	      if (ui_precount) ui_precount--;
 	      if (ui_precount == 0)
	       {
	          ui_precount = ui_rampe;
              //Berechnung Vorteiler fuer Geschwindigkeit
	          if (uc_motorstate == 1)
	           {
		          ui_vist += ui_vsoll /ui_kp1; 
				  if (ui_vist < ui_vmin)  ui_vist = ui_vmin;
			      if (ui_vist > ui_vsoll) ui_vist = ui_vsoll;
			      if (ui_vist > ui_vmax)  ui_vist = ui_vmax;
	           }
			
    	      if (uc_motorstate == 2)
	           {
		          if (ui_vist > ui_vmin) ui_vist -= ui_vsoll/ui_kp1;
			      if (ui_vist < ui_vmin) ui_vist = ui_vmin;
    	       }						
		   
		      if (ui_vist > 0x00DF)
		       {
		          TCLK = ((unsigned int) ui_vist & 0x00FF) - 0x0020;
			      TPRE = 0x02;
		       }
		       else {
		               TCLK = ui_vist;
				       TPRE = 0x03;
		            }
		   }		      	
	   }   
    }
}


//*****************************************************************************
//	MOTOR - INIT
//  Input:	-
//	Output:	-
//*****************************************************************************
void MOTOR_INIT(void)
{
   watchdog_reset();
   PORTS_INIT();
   uc_notaus = 1;  //wird durch Referenzfahrt zurueckgesetzt
   sl_stepcount = 0;
   sl_sollpos = 0;
   ui_stepsproumdrehung = STEPS_U;
   uc_referenzfahrt = 0;
   uc_referenzrichtung = 0;
   uc_richtung = 0;
   uc_motorstate = 0;
   ui_precount = 0;
   ui_kp = KP;
   ui_vmax  = MOTOR_Drehzahl2INC(VMAX);
   ui_vsoll = MOTOR_Drehzahl2INC(VMAX);
   ui_vsollwert = VMAX;
   ui_vmin  = MOTOR_Drehzahl2INC(VMIN);
   ui_vist  = 0;
   
   SREG &= 0x7F;
  				    //Timer init fuer 1ms = 16MHz / VT32 / CNT50
   TIMSK |= 0x40;  //Timer2 Int enable
   TCCR2 = TPRE;
   TCNT2 = TCLK;
   TIFR |= 0x40;	//Overflow Flag reset
   SREG |= 0x80;	//global Int enable

   MOTOR_MODE(8);  //Achtelschritt
   MOTOR_PARAMETER_KP(ui_kp);
}


//*****************************************************************************
//	MOTOR - MODE
//  Input:	1,2,4,8 Schritte pro Vollschritt
//	Output:	-
//*****************************************************************************
void MOTOR_MODE(unsigned char uc_steps)
{
   watchdog_reset();

   switch(uc_steps)
   {
   case 1:   PORTS_MODE(0);uc_step = 1;break;
   case 2:   PORTS_MODE(1);uc_step = 2;break;
   case 4:   PORTS_MODE(2);uc_step = 4;break;
   case 8:   PORTS_MODE(3);uc_step = 8;break;
   default:  PORTS_MODE(0);uc_step = 1;break;
   }  
}


//*****************************************************************************
//	MOTOR - NewStep
//  Input:	-
//	Output:	-
//*****************************************************************************
void MOTOR_STEPCOUNT(void)
{
   if (uc_richtung)
    {   //Linkslauf
	  if (uc_referenzrichtung) sl_stepcount++;
	                      else sl_stepcount--;
	}
	else {  //Rechtslauf
            if (uc_referenzrichtung) sl_stepcount--;
	                            else sl_stepcount++;
	     }
}


//*****************************************************************************
//	MOTOR - Richtung
//  Input:	0/1 Richtung
//	Output:	-
//*****************************************************************************
void MOTOR_RICHTUNG(unsigned char uc_drehrichtung)
{
   if (uc_referenzrichtung)
    {
       if (uc_drehrichtung)
        {
	       uc_richtung = 1;
	  	   PORTS_DIR(1);
	    }
     	else {
	            uc_richtung = 0;
                PORTS_DIR(0);
     	     }

	}
	else {
            if (uc_drehrichtung)
             {
          	    uc_richtung = 0;
                PORTS_DIR(0);
             }
             else {
	                 uc_richtung = 1;
         			 PORTS_DIR(1);
         	      }
		 }		 
}


//*****************************************************************************
//	MOTOR - ON
//  Input:	-
//	Output:	-
//*****************************************************************************
void MOTOR_ON(void)
{
   PORTS_POWER(1);
   PORTS_ENABLE(1);
}


//*****************************************************************************
//	MOTOR - ON
//  Input:	0/1 Richtung
//	Output:	-
//*****************************************************************************
void MOTOR_OFF(void)
{
   PORTS_ENABLE(0);
   PORTS_POWER(0);
   uc_motorstate = 0;
}


//*****************************************************************************
//	MOTOR - Bremse
//  Input:	0/1 on /off
//	Output:	-
//*****************************************************************************
void MOTOR_BRAKE(unsigned char uc_x)
{
   if (uc_x) PORTS_BRAKE(1); else PORTS_BRAKE(0);
}


//*****************************************************************************
//	MOTOR - Drive
//  Input:	-
//	Output:	-
//*****************************************************************************
void MOTOR_START(unsigned int ui_v)
{
   watchdog_reset();
   ui_vsoll = MOTOR_Drehzahl2INC(ui_v);
   ui_vist = 0;
   MOTOR_ON();
   watchdog_reset();
   MOTOR_BRAKE(0);
   timer_delay(50);
   ui_precount = ui_rampe;
   uc_motorstate = 1;   
}


//*****************************************************************************
//	MOTOR - Drive
//  Input:	Sollposition
//	Output:	-
//*****************************************************************************
unsigned char MOTOR_DRIVE(long sl_pos)
{
long sl_rampe,sl_stufen;
unsigned int ui_vstep;
unsigned char uc_erg = 0;

   watchdog_reset();
   if ((!uc_motorstate) && (!uc_notaus))
    {
       if (sl_pos != sl_stepcount)
        {
	       if (sl_pos < sl_stepcount) MOTOR_RICHTUNG(0); 
	                             else MOTOR_RICHTUNG(1);	
							 						 
		   watchdog_reset();
           sl_sollpos = sl_pos;  
		   ui_vstep = ui_kp1 / ui_rampe;
		   watchdog_reset();
           ui_vsoll = MOTOR_Drehzahl2INC(ui_vsollwert);
		      
           sl_stufen = ((long) ui_vsoll / (long) ui_vstep );
		   sl_rampe = sl_stufen * (long) ui_rampe;	   	   
		   watchdog_reset();

    	   if (sl_sollpos > sl_stepcount)
	        {
		       sl_bremsrampe = sl_sollpos - sl_rampe;
 		       watchdog_reset();
		       sl_delta = (sl_sollpos - sl_stepcount) /2;
		       if (sl_delta < sl_rampe) sl_bremsrampe = sl_delta;
        	}   
	        else {
			        sl_bremsrampe = sl_sollpos + sl_rampe;			   
 		            watchdog_reset();
                    sl_delta = (sl_stepcount - sl_sollpos) /2;
 		            if (sl_delta < sl_rampe) sl_bremsrampe = sl_delta;
			     }
		   watchdog_reset();
     	   MOTOR_START(ui_vsollwert);	//und los gehts
		}   					 							 							 
       uc_erg = 1;
	}
   return(uc_erg);		
}


//*****************************************************************************
//	MOTOR - Drive to Referenz
//  Input:	- Referenzrichtung
//	Output:	-
//*****************************************************************************
void MOTOR_REFERENZFAHRT(void)
{
   watchdog_reset();
   uc_notaus = 0;
   sl_stepcount = 0;	 	 
   sl_sollpos = 0;
   uc_referenzfahrt = 2;  
   uc_refstatus = 1; //Start der Referenzfahrt   
}


//*****************************************************************************
//	MOTOR - Drive to Referenz
//  Input:	- Referenzrichtung
//	Output:	-
//*****************************************************************************
void MOTOR_REFERENZRICHTUNG(unsigned char uc_x)
{
   if (uc_x) uc_referenzrichtung = 1; else uc_referenzrichtung = 0;
   uc_notaus = 1; //erzwinge Referenzfahrt
}

   	
//*****************************************************************************
//	MOTOR - Drehzahl
//  Input:	-
//	Output:	Position
//*****************************************************************************
void MOTOR_DREHZAHL(unsigned int ui_n)
{
   ui_vsollwert = MOTOR_Drehzahl2INC(ui_n);
}


//*****************************************************************************
//	MOTOR - GetDrehzahl
//  Input:	-
//	Output:	Position
//*****************************************************************************
unsigned int MOTOR_GETDREHZAHL(void)
{
   return(MOTOR_INC2Drehzahl(ui_vist));
}

   
//*****************************************************************************
//	MOTOR - Get Position
//  Input:	-
//	Output:	Position
//*****************************************************************************
long MOTOR_POSITION(void)
{
   return(sl_stepcount);
}


//*****************************************************************************
//	MOTOR - Zustand
//  Input:	-
//	Output:	ungleich 1 => in Fahrt
//*****************************************************************************
unsigned char MOTOR_ZUSTAND(void)
{
   return(uc_motorstate);
}


//*****************************************************************************
//	MOTOR - PARAMETER KP
//  Input:	kp
//	Output:	-
//*****************************************************************************
void MOTOR_PARAMETER_KP(unsigned int ui_param_kp)
{
   watchdog_reset();
   ui_kp = ui_param_kp;
   if (ui_kp > 25500) ui_kp = 1;
   ui_rampe = ui_kp / 255;

   watchdog_reset();
   ui_kp1 = ui_kp - ui_rampe * 255;

   if ((ui_rampe < 1) && (ui_rampe > 99)) ui_rampe = 1;
   watchdog_reset();
   ui_rampe = 100 - ui_rampe;
   
   if (ui_kp1 < 1) ui_kp1 = 1;  
   watchdog_reset();
}


//*****************************************************************************
//	MOTOR - Drehzahl in INC
//  Input:	v
//	Output:	-
//*****************************************************************************
unsigned int MOTOR_Drehzahl2INC(unsigned int ui_n)
{
long ul_x,ul_freq1,ul_freq2;

   if (ui_n == 0) return(0);
   watchdog_reset();
   
   //Taktfrequenz fuer TIMERREGISTER
   ul_freq1 = PROZESSOR_CLK / 16;   //Vorteiler  8 *2 ... *2 wegen 0/1-Wechsel in Int-routine
   ul_freq2 = PROZESSOR_CLK / 128;  //Vorteiler 64 *2
   watchdog_reset();

   //TPRE = 3
   ul_x = ul_freq2 * 60 / (long) ui_stepsproumdrehung / (long) ui_n;
   ul_x = 256 - ul_x;
   watchdog_reset();
   
   //TPRE = 2
   if (ul_x > 0xDF) //223
    {
      ul_x = ul_freq1 * 60 / (long) ui_stepsproumdrehung / (long) ui_n;
      watchdog_reset();
      ul_x = 256 - ul_x;
	  ul_x += 0x000000E0;	
	}

   watchdog_reset();
   return(ul_x);
}


//*****************************************************************************
//	MOTOR - INC in Drehzahl
//  Input:	v
//	Output:	-
//*****************************************************************************
unsigned int MOTOR_INC2Drehzahl(unsigned int ui_inc)
{
long ul_x,ul_inc,ul_freq1,ul_freq2;

   if (ui_inc == 0) return(0);
   watchdog_reset();
   
   //Taktfrequenz fuer TIMERREGISTER
   ul_freq1 = PROZESSOR_CLK / 16;   //Vorteiler  8 *2 wegen 0/1-Wechsel in Int-routine
   ul_freq2 = PROZESSOR_CLK / 128;  //Vorteiler 64 *2

   watchdog_reset();
   if (ui_inc > 0xDF)
    {    //TPRE = 2
	  ul_inc = (long) ui_inc - 0x000000E0;
      watchdog_reset();
	  ul_inc = 256 - ul_inc;
      watchdog_reset();
      ul_x = ul_freq1 * 60 / (long) ui_stepsproumdrehung / ul_inc;
	}
    else { //TPRE = 3
 		    ul_inc = 256 - ui_inc;
 		    watchdog_reset();
            ul_x = ul_freq2 * 60 / (long) ui_stepsproumdrehung / ul_inc;
         }

   watchdog_reset();		 	 
   return(ul_x);   
}


//*****************************************************************************
//	MOTOR - PARAMETER v
//  Input:	v
//	Output:	-
//*****************************************************************************
void MOTOR_PARAMETER_VMIN(unsigned int ui_param_v)
{
   if (ui_param_v < VMIN) ui_vmin = MOTOR_Drehzahl2INC(VMIN);
                     else ui_vmin = MOTOR_Drehzahl2INC(ui_param_v);
}


//*****************************************************************************
//	MOTOR - PARAMETER v
//  Input:	v
//	Output:	-
//*****************************************************************************
void MOTOR_PARAMETER_VMAX(unsigned int ui_param_v)
{
   if (ui_param_v > VMAX) ui_vmax = MOTOR_Drehzahl2INC(VMAX);
                     else ui_vmax = MOTOR_Drehzahl2INC(ui_param_v);
}




