//*****************************************************************************
//
//	MT701 Ansteuerport + Endlagenschalter
//
//  PORTD.2			   Bremse
//  PORTD.3			   M0
//  PORTD.4			   M1
//  PORTD.5			   Enable
//  PORTD.6			   Direction
//  PORTD.7			   Step
//
//  PORTE.6			   Endlage 1			   
//  PORTE.5			   Endlage 2			   
//
//*****************************************************************************

#include <iom128v.h>
#include "m128int.h"
#include "ports.h"
#include "zustand.h"
#include "motor.h"


#pragma interrupt_handler PORTS_ENDLAGE1_INT:EXT_INT6
#pragma interrupt_handler PORTS_ENDLAGE2_INT:EXT_INT5

#define ENDSTUFE

extern unsigned char uc_referenzrichtung;
extern unsigned char uc_referenzfahrt;
extern long sl_stepcount;
extern unsigned char uc_notaus;

//*****************************************************************************
//	PORTS - INIT
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_INIT(void)
{
  DDRD  = 0xFC;
  PORTD = 0x20;    //Motor disable
  DDRE  &= 0x8F;
  DDRE  |= 0x04;
  PORTE |= 0x70;
  
  EICRB &= 0xC2;  
  EICRB |= 0x3C;  //rising Edge // 0x28;  //falling EDGE   
  EIMSK |= 0x60;  //ext.int freigeben
  SREG  |= 0x80;  //INT Freigabe global  
}


//*****************************************************************************
//	PORTS - Endlage 1
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_ENDLAGE1_INT(void)
{
   EIFR |= 0x40;
   if (uc_referenzfahrt)
    {
	  if (uc_referenzfahrt == 1)
	   {
	     sl_stepcount = 0;
	     uc_referenzfahrt = 2;
	   }	  
	} 
	else {
	       MOTOR_OFF();
		   uc_notaus = 1;
		 }    
}


//*****************************************************************************
//	PORTS - Endlage 2
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_ENDLAGE2_INT(void)
{
   EIFR |= 0x20;
   if (uc_referenzfahrt)
    {
	  if (uc_referenzfahrt == 1)
	   {
	     sl_stepcount = 0;
	     uc_referenzfahrt = 2;
	   }	  
	}  
	else {
	       MOTOR_OFF();
		   uc_notaus = 1;
		 }    
}


//*****************************************************************************
//	PORTS - SET 
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_SET(unsigned char uc_db)
{
  PORTD = uc_db;
}


//*****************************************************************************
//	PORTS - GET 
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char PORTS_GET(void)
{
  return(PIND);
}



//*****************************************************************************
//	PORTS - CLK
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_CLK(unsigned char uc_x)
{
#ifdef ENDSTUFE
  if (!uc_x) PORTS_SET(PORTS_GET() | 0x80);
       else PORTS_SET(PORTS_GET() & 0x7F);
#endif
#ifndef ENDSTUFE
  if (uc_x) PORTS_SET(PORTS_GET() | 0x80);
       else PORTS_SET(PORTS_GET() & 0x7F);
#endif	   
}


//*****************************************************************************
//	PORTS - DIR
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_DIR(unsigned char uc_x)
{
#ifdef ENDSTUFE
  if (!uc_x) PORTS_SET(PORTS_GET() | 0x40);
        else PORTS_SET(PORTS_GET() & 0xBF);
#endif
#ifndef ENDSTUFE
  if (uc_x) PORTS_SET(PORTS_GET() | 0x40);
       else PORTS_SET(PORTS_GET() & 0xBF);
#endif	   
}


//*****************************************************************************
//	PORTS - Enable
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_ENABLE(unsigned char uc_x)
{
#ifdef ENDSTUFE
  if (!uc_x) PORTS_SET(PORTS_GET() & 0xDF);
        else PORTS_SET(PORTS_GET() | 0x20);
#endif
#ifndef ENDSTUFE
  if (uc_x) PORTS_SET(PORTS_GET() & 0xDF);
       else PORTS_SET(PORTS_GET() | 0x20);
#endif	   
}


//*****************************************************************************
//	PORTS - Mode
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_MODE(unsigned char uc_x)
{
  PORTS_SET(PORTS_GET() & 0xE7);
  uc_x = (uc_x & 0x03) << 3;
  PORTS_SET(PORTS_GET() | uc_x);
}


//*****************************************************************************
//	PORTS - Power
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_POWER(unsigned char uc_x)
{
  if (uc_x) PORTS_SET(PORTS_GET() | 0x04);
       else PORTS_SET(PORTS_GET() & 0xFB);
}


//*****************************************************************************
//	PORTS - Brake
//  Input:	-
//	Output:	-
//*****************************************************************************
void PORTS_BRAKE(unsigned char uc_x)
{
  if (uc_x) PORTE |= 0x04;
       else PORTE &= 0xFB;
}



