//*****************************************************************************
//
//	Temperaturmessung mit LM75
// 
//*****************************************************************************
#include <iom128v.h>
#include "m128int.h"
#include "i2ch.h"
#include "thermo.h"


/*****************************************************************************/
//  Word in LM75 schreiben
//  Input:  I2C-Adr,Register,Daten
//  Output: -
/*****************************************************************************/

void TEMP_write_configregister(unsigned char uc_adr,unsigned char uc_daten)
{
   I2C_constart();
   if (I2C_ERROR()) return; 
   I2C_transbyteADR(uc_adr & 0xFE);
   if (I2C_ERROR()) return;    
   I2C_transbyte(0x01);                          
   if (I2C_ERROR()) return;
   I2C_transbyte(uc_daten);                          
   if (I2C_ERROR()) return;
   I2C_constop();
   if (I2C_ERROR()) return;   
}


/*****************************************************************************/
//  Word aus LM75 lesen
//  Input:  I2C-Adr,Register
//  Output: Daten
/*****************************************************************************/

unsigned char TEMP_read_configregister(unsigned char uc_adr)
{
unsigned char uc_x = 0;

   I2C_constart();
   if (I2C_ERROR()) return(0);
   I2C_transbyteADR(uc_adr & 0xfe);
   if (I2C_ERROR()) return(0);
   I2C_transbyte(0x01);
   if (I2C_ERROR()) return(0);
   I2C_constart();
   if (I2C_ERROR()) return(0);
   I2C_transbyteADR(uc_adr | 1);
   if (I2C_ERROR()) return(0);
   uc_x = I2C_recbyteend();
   if (I2C_ERROR()) return(0);
   I2C_constop();
   if (I2C_ERROR()) return(0);
   return(uc_x);
}


/*****************************************************************************/
//  Word in LM75 schreiben
//  Input:  I2C-Adr,Register,Daten
//  Output: -
/*****************************************************************************/

void TEMP_write_register(unsigned char uc_adr,unsigned char uc_reg,unsigned int ui_daten)
{
   I2C_constart();
   if (I2C_ERROR()) return; 
   I2C_transbyteADR(uc_adr & 0xFE);
   if (I2C_ERROR()) return;    
   I2C_transbyte(uc_reg);                          
   if (I2C_ERROR()) return;
   I2C_constop();
   if (I2C_ERROR()) return;
   
   I2C_constart();
   if (I2C_ERROR()) return; 
   I2C_transbyteADR(uc_adr & 0xFE);
   if (I2C_ERROR()) return;    
   I2C_transbyte((unsigned int) ui_daten >> 8);                          
   if (I2C_ERROR()) return;
   I2C_transbyte(ui_daten);                          
   if (I2C_ERROR()) return;
   I2C_constop();
   if (I2C_ERROR()) return;   
}


/*****************************************************************************/
//  Word aus LM75 lesen
//  Input:  I2C-Adr,Register
//  Output: Daten
/*****************************************************************************/

unsigned int TEMP_read_register(unsigned char uc_adr,unsigned char uc_reg)
{
unsigned int ui_x = 0;

   I2C_constart();
   if (I2C_ERROR()) return(0);
   I2C_transbyteADR(uc_adr & 0xfe);
   if (I2C_ERROR()) return(0);
   I2C_transbyte(uc_reg);
   if (I2C_ERROR()) return(0);
   I2C_constop();
   if (I2C_ERROR()) return(0);
   
   I2C_constart();
   if (I2C_ERROR()) return(0);
   I2C_transbyteADR(uc_adr | 1);
   if (I2C_ERROR()) return(0);
   ui_x = (unsigned int) I2C_recbyte() << 8;
   if (I2C_ERROR()) return(0);
   ui_x = ui_x | I2C_recbyteend();
   if (I2C_ERROR()) return(0);
   I2C_constop();
   if (I2C_ERROR()) return(0);
   return(ui_x);
}


/*****************************************************************************/
//  Temperatur aus LM75 lesen
//  Input:  I2C-Adr
//  Output: (Temperatur * 10)  Wert >= 0x8000 neg. Temperatur
/*****************************************************************************/

int TEMP_temperatur(unsigned char uc_nr)
{
unsigned char uc_Slaveadr = THERMOSENSOR;
int si_temperatur;

  uc_Slaveadr = uc_Slaveadr | ((uc_nr & 0x03) << 1);
  
  si_temperatur = (int) TEMP_read_register(uc_Slaveadr,THERMOREG_TEMPERATUR);
  si_temperatur = si_temperatur >> 7;
  
  if (si_temperatur > 0x100)
   {
      si_temperatur = 0x200 - si_temperatur;
	  si_temperatur = si_temperatur * 5;
	  si_temperatur = si_temperatur | 0x8000; 
   } 
   else si_temperatur = si_temperatur * 5;
                    
  return(si_temperatur);
}  


